<?php


/////////////////////////////////////////////////////////////////////////////////////////////////////

//GC Color Theme
function additional_admin_color_schemes() {
  //Get the theme directory
  $theme_dir = get_template_directory_uri();
 
  //GoingClear
 wp_admin_css_color(
   'goingclear',
   __('GoingClear'),
   admin_url("css/colors/goingclear/colors.css"),
   array('#8ec652', '#008cc6', '#e5e5e5', '#999'),
   array( 'base' => '#e5f8ff', 'focus' => '#fff', 'current' => '#fff' )
 );
}
add_action('admin_init', 'additional_admin_color_schemes');


//Reset GC Color Theme as Default for New Users
function set_default_admin_color($user_id) {
  $args = array(
    'ID' => $user_id,
    'admin_color' => 'goingclear'
  );
  wp_update_user( $args );
}
add_action('user_register', 'set_default_admin_color');


/////////////////////////////////////////////////////////////////////////////////////////////////////

//Remove Login Wiggle
function remove_login_shake() {
  // remove the wp_shake JavaScript
  remove_action( 'login_head', 'wp_shake_js', 12 );
}
add_action( 'login_head', 'remove_login_shake' );


function custom_header() { 
    echo '<link rel="stylesheet" type="text/css" href="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/login.css" />'; 
    echo '<script type="text/javascript" src="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/jquery-3.2.1.min.js"></script>';
    echo '<script type="text/javascript" src="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/login.js"></script>';
	 
}
add_action('login_head', 'custom_header');

/////////////////////////////////////////////////////////////////////////////////////////////////////


if ( ! function_exists( 'lawyer_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 */
	function lawyer_setup() {
		/*
		 * Make theme available for translation.
		 */
		load_theme_textdomain( 'lawyer', get_template_directory() . '/languages' );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Let WordPress manage the document title.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 */
		add_theme_support( 'post-thumbnails' );
		add_image_size( 'post_737x361', 737, 361, true );
		add_image_size( 'post_156x150', 165, 150, true );
		add_image_size( 'attorney_545x543', 545, 543, true );
		add_image_size( 'practice_737x432', 737, 432, true );
		add_image_size( 'practice_thumb_545x381', 545, 381, true );

		// This theme uses wp_nav_menu() in one location.
		register_nav_menus( array(
			'menu-1' => esc_html__( 'Primary menu', 'lawyer' ),
			'menu-2' => esc_html__( 'Footer menu 1', 'lawyer' ),
			'menu-3' => esc_html__( 'Footer menu 2', 'lawyer' ),
		) );

	}
endif;
add_action( 'after_setup_theme', 'lawyer_setup' );


/**
 * Enqueue scripts and styles.
 */
function lawyer_scripts() {
	
	/**
	 * Enqueue styles.
	 */
	wp_enqueue_style('adobe-fonts', 'https://use.typekit.net/gzo7swr.css', array(), false, 'all');
	wp_enqueue_style('plugins', get_template_directory_uri() . '/css/plugins.css', array(), false, 'all');
	wp_enqueue_style( 'lawyer-style', get_stylesheet_uri() );

	/**
	 * Enqueue scripts.
	 */
	wp_enqueue_script('jquery');
	wp_enqueue_script('plugins', get_template_directory_uri() . '/js/plugins.js', array(), false, true);
	$gmap_api = get_field('google_map_api_key', 'options');
	$googleapi = "https://maps.googleapis.com/maps/api/js?key=$gmap_api";
	wp_enqueue_script('gmap_api', $googleapi, array(), false, true);
	wp_enqueue_script('scripts', get_template_directory_uri() . '/js/scripts.js', array(), false, true);

	$map_icon = get_field('map_pin', 'options');
	$google_map = get_field('google_map');

	global $wp_query;
	$data = array(
		'gmap_latitude' => $google_map['lat'],
		'gmap_longitude' => $google_map['lng'],
		'gmap_address' => $google_map['address'],
		'map_icon' => $map_icon,
		'site_url' => get_template_directory_uri(),
		'admin_ajax'   => admin_url( 'admin-ajax.php' ),
		'query' => $wp_query->query,
	);
	wp_localize_script('scripts', 'ajax', $data);
}
add_action( 'wp_enqueue_scripts', 'lawyer_scripts' );


/**
 * Register and enqueue a custom stylesheet in the WordPress admin.
 */
function lemonade_scripts_admin() {
    wp_enqueue_style('icon-fonts', get_template_directory_uri() . '/css/icon_fonts.css', array(), false, 'all');
}
add_action( 'admin_enqueue_scripts', 'lemonade_scripts_admin' );

/**
 * Implement the Custom Header feature.
 */
require get_template_directory() . '/navwalker.php';


/**
 * ACF OPTIONS PAGE
 *
 */
if(function_exists('acf_add_options_page')) {
    acf_add_options_page();
}

/**
 * Dashboard google map api key support.
 */
add_filter('acf/settings/google_api_key', function () {
	$gmap_api = get_field('google_map_api_key', 'options');
    return $gmap_api;
});

function hex2rgb($hex) {
   $hex = str_replace("#", "", $hex);

   if(strlen($hex) == 3) {
      $r = hexdec(substr($hex,0,1).substr($hex,0,1));
      $g = hexdec(substr($hex,1,1).substr($hex,1,1));
      $b = hexdec(substr($hex,2,1).substr($hex,2,1));
   } else {
      $r = hexdec(substr($hex,0,2));
      $g = hexdec(substr($hex,2,2));
      $b = hexdec(substr($hex,4,2));
   }
   return 'rgba('.$r.', '.$g.', '.$b.', 0.8)';
}

/**
 * Get all page id
 *
 */ 
function getPageID() {
	global $post;
	$postid = $post->ID;
	if(is_home() && get_option('page_for_posts')) {
		$postid = get_option('page_for_posts');
	}

	return $postid;
}

/* Customized header title */
function customizedtitle($title){
	if(is_category() || is_author()){
		return get_the_archive_title();
	} else if( is_search()){
		return sprintf( esc_html__( 'Search Results for: %s', 'lemonade' ), '<strong>' . get_search_query() . '</strong>' ); 
	}
	return $title;
}

/* Limit blog text*/
function Limit_Text($x, $length) {
	if(strlen($x)<=$length) {
		echo $x;
	} else {
		$y=substr($x,0,$length) . '...';
		echo $y;
	}
}

/* ACF not working use jquery */
function hide_editor_custom_js() {
    echo '<script type="text/javascript">
			jQuery(document).ready(function(){
				jQuery("#page_template").change( function() {
					jQuery("#_home_page_options").hide();
					jQuery("#postdivrich").show();
					switch( jQuery( this ).val() ) {
						case "t_home.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_about.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_practice.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_contact.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;
					}
				}).change();
			});
		</script>';
}
add_action('admin_head', 'hide_editor_custom_js');

/* Title After ACF Field */
function my_acf_admin_head() {

	$screen = get_current_screen();
	if ($screen->post_type = 'page' ) {
		
	echo '<script type="text/javascript">
		    (function($) {
		        
		        $(document).ready(function(){
		            
		            $("#acf-group_5a34004f012c1").append( $("#postdivrich") );
		            
		        });
		        
		    })(jQuery);    
	    </script>';
	}
}
add_action('acf/input/admin_head', 'my_acf_admin_head');


/**
 * Gravity form user role
 *
 */
add_filter( 'gform_confirmation_anchor', '__return_false' );
function gforms_editor_access() {
  
  $role = get_role( 'editor' );
  $role->add_cap( 'gform_full_access' );
}
add_action( 'after_switch_theme', 'gforms_editor_access' );

function login_logo_url_title() {
 	return 'Lawyer Theme';
}
add_filter( 'login_headertitle', 'login_logo_url_title' );

function login_logo_url() {
	return get_bloginfo( 'url' );
}
add_filter( 'login_headerurl', 'login_logo_url' );

function remove_admin_menus() {
	//remove_menu_page( 'themes.php' );               			// Appearance
	remove_menu_page( 'edit-comments.php' );      				// Comments
	//remove_menu_page( 'users.php' );                			// Users
	remove_menu_page( 'tools.php' );                			// Tools
	//remove_menu_page( 'options-general.php' );      			// Settings
	//remove_menu_page( 'plugins.php' );              			// Plugins
	remove_menu_page( 'sharethis-general' );					// share this
	//remove_menu_page( 'edit.php?post_type=acf-field-group' );	// Custom Field 
	remove_menu_page( 'pods' );               					// Pods Custom post type
}
add_action( 'admin_menu', 'remove_admin_menus', 999);

function reorder_admin_menu( $__return_true ) {
    return array(
         'index.php', 							// Dashboard
         'separator1', 							// --Space--
         'acf-options',							// ACF Theme Settings
         'edit.php?post_type=page', 			// Pages 
         'edit.php', 							// Posts
         'separator2', 							// --Space--
         'gf_edit_forms',						// Gravity Forms
         'upload.php', 							// Media
         'wpseo_dashboard',						// Yoasta
         'edit.php?post_type=attorneys', 		// attorneys
         'edit.php?post_type=practice_areas', 	// practice_areas
         'gadash_settings',						// Google Analytics
         'themes.php', 							// Appearance
         'edit-comments.php', 					// Comments 
         'users.php', 							// Users
         'tools.php', 							// Tools
         'options-general.php', 				// Settings
         'plugins.php', 						// Plugins
   );
}
add_filter( 'custom_menu_order', 'reorder_admin_menu' );
add_filter( 'menu_order', 'reorder_admin_menu' );


/**
 * ajaxify blog post filter
 *
 */
add_action('wp_ajax_nopriv_filterposts', 'filterposts_func');
add_action('wp_ajax_filterposts', 'filterposts_func');

function filterposts_func(){
	$args = isset( $_POST['query'] ) ? array_map( 'esc_attr', $_POST['query'] ) : array();
	$args['post_type'] = isset( $args['post_type'] ) ? esc_attr( $args['post_type'] ) : 'post';
	$args['paged'] = esc_attr( $_POST['page'] );
	$args['post_status'] = 'publish';

	$posts = new WP_Query($args);

	global $wp_query;

	if($posts->have_posts()) : while ($posts->have_posts()) : $posts->the_post();
	    $featured_image = get_field('blog_post_featured_image'); 
	    $featured_image = $featured_image ? $featured_image['sizes']['post_737x361'] : get_template_directory_uri().'/images/placeholder-blog.jpg'; 
	?>

	<article id="post-<?php the_ID(); ?>" <?php post_class('blog-post col-md-6 col-sm-6 col-xs-12'); ?>>
	    <div class="media">
	        <a href="<?php the_permalink(); ?>">
				<img src="<?php echo $featured_image; ?>" class="img-responsive" alt="<?php the_title(); ?>">
			</a>
	        <h3 class="entry-meta"><?php the_time('j'); ?><span class="month"><?php the_time('M'); ?></span></h3>
	    </div>
	    <div class="entry-content">
	    	<a href="<?php the_permalink(); ?>"><h3 class="entry-title"><?php Limit_Text(get_the_title(), 49); ?></h3></a>
	    	<div class="posted-on">
	    		<span class="posted-by">By <a href="<?php echo get_author_posts_url( get_the_author_meta( 'ID' ), get_the_author_meta( 'user_nicename' ) ); ?>"><?php the_author(); ?></a></span> | <span class="post-date"><?php echo get_the_date(); ?></span>
	    	</div>
	        <p><?php Limit_Text(get_the_content(), 170); ?></p>
	    </div>
	</article><!-- .article -->

<?php endwhile; else: ?>

    <div class="entry-content notResult col-md-12 col-sm-12 col-xs-12">
        <h2 class="no-content text-center" style="padding: 50px 0">
            <?php _e('No more posts!!!', 'lawyer'); ?>
        </h2>
    </div>  

<?php 
	endif; 
	wp_die();
}


